import { Socket } from "socket.io-client";
import { Sushi } from "./Ressources.js";

class Player {
  image = new Image();
  name: string;
  id: number;
  x: number;
  y: number;
  radius: number;
  speed: number;
  direction: { x: number; y: number };
  imageId: number;
  imageFolder: string;
  isDead: boolean;
  constructor(
    x: number,
    y: number,
    radius: number,
    imageFolder: string,
    id: number,
    imageId: number,
    name: string
  ) {
    this.name = name;
    this.id = id;
    this.x = x;
    this.y = y;
    this.radius = radius;
    this.speed = 4;
    this.direction = {
      x: 0,
      y: 0,
    };
    this.imageId = imageId;
    this.imageFolder = imageFolder;
    this.image.src = `/images/character/${this.imageFolder}/${this.imageFolder}${this.imageId}.png`;
    this.isDead = false;
  }
  draw(ctx: CanvasRenderingContext2D) {
    // Dessiner le cercle
    ctx.beginPath();
    ctx.drawImage(
      this.image,
      this.x - this.radius,
      this.y - this.radius,
      this.radius * 2,
      this.radius * 2
    );
    // Dessiner le texte
    ctx.fillStyle = "#0094FF";
    ctx.textAlign = "center";
    ctx.textBaseline = "middle";
    ctx.font = `bold ${this.radius / 2}px Arial`;
    ctx.fillText(this.name, this.x, this.y);
    ctx.arc(this.x, this.y, this.radius, 0, 2 * Math.PI);
    ctx.strokeStyle = "black";
    ctx.stroke();
  }

  grow() {
    this.radius += 0.5;
    if (this.radius % 80 === 0 && this.imageId <= 5) {
      this.imageId += 1;
      this.image.src = `/images/character/${this.imageFolder}/${this.imageFolder}${this.imageId}.png`;
    }
  }

  eatSushi(sushis: Sushi[], socket: Socket) {
    sushis.forEach((sushi: Sushi, idx) => {
      const dx = sushi.x - this.x;
      const dy = sushi.y - this.y;
      const distance = Math.sqrt(dx * dx + dy * dy);
      if (distance < this.radius && !sushi.drop && !this.isDead) {
        sushi.drop = true;
        socket.emit("updateSushi", idx);
        this.grow();
      }
    });
  }
}

export default Player;
