import Player from "./Player";
import Router from "./Router";
import { Sushi } from "./Ressources";
import { io, Socket } from "socket.io-client";

const Main = (playerName: string, playerCharacter: string) => {
  const canvas = document.querySelector(".map") as HTMLCanvasElement;
  const ctx = canvas.getContext("2d") as CanvasRenderingContext2D;

  canvas.width = document.documentElement.clientWidth;
  canvas.height = document.documentElement.clientHeight;

  const ctxWidth: number = 6000;
  const ctxHeight: number = 3340;
  let xStart = canvas.width / 2;
  let xEnd = ctxWidth - canvas.width / 2;
  let yStart = canvas.height / 2;
  let yEnd = ctxHeight - canvas.height / 2;

  window.addEventListener("resize", (e) => {
    e.preventDefault();
    canvas.width = document.documentElement.clientWidth;
    canvas.height = document.documentElement.clientHeight;
    xStart = canvas.width / 2;
    xEnd = ctxWidth - canvas.width / 2;
    yStart = canvas.height / 2;
    yEnd = ctxHeight - canvas.height / 2;
  });

  const backgroundImage = new Image();
  backgroundImage.src = "/images/map.jpg";

  function youAreDead(player: Player) {
    // Récupération des dimensions du canvas
    let canvasWidthI = canvas.width;
    let canvasHeightI = canvas.height;

    // Calcul des dimensions de l'interface
    let interfaceWidth = Math.round(canvasWidthI * 0.7);
    let interfaceHeight = Math.round(canvasHeightI * 0.7);

    // Positionnement de l'interface au centre du canvas
    /*let interfaceLeft = Math.round((canvasWidth - interfaceWidth) / 2);
    let interfaceTop = Math.round((canvasHeight - interfaceHeight) / 2);*/
    let interfaceLeft = player.x - interfaceWidth / 2;
    let interfaceTop = player.y - interfaceHeight / 2;

    // Dessin de la bordure orange de l'interface
    ctx.lineWidth = 10;
    ctx.strokeStyle = "#D84800";
    ctx.strokeRect(
      interfaceLeft,
      interfaceTop,
      interfaceWidth,
      interfaceHeight
    );

    // Dessin du fond transparent de l'interface
    ctx.fillStyle = "#00000048";
    ctx.fillRect(interfaceLeft, interfaceTop, interfaceWidth, interfaceHeight);

    // Ajout du titre "Welcome in heaven"
    ctx.font = "bold 45px Arial";
    ctx.textAlign = "center";
    ctx.fillStyle = "#D84800";
    ctx.fillText(
      "Welcome in heaven",
      interfaceLeft + interfaceWidth / 2,
      interfaceTop + interfaceHeight * 0.4
    );

    // Ajout du bouton "Quitter"
    ctx.fillStyle = "#D84800";
    ctx.fillRect(
      interfaceLeft + interfaceWidth / 2 - 50,
      interfaceTop + interfaceHeight * 0.6,
      100,
      50
    );
    ctx.font = "bold 25px Arial";
    ctx.textAlign = "center";
    ctx.fillStyle = "#fff";
    ctx.fillText(
      "Quitter",
      interfaceLeft + interfaceWidth / 2,
      interfaceTop + interfaceHeight * 0.65
    );

    const routes = [
      { path: "/map", page: "map" },
      { path: "/", page: "home" },
    ];

    let router = new Router(routes);

    // Ajout de l'événement de clic sur le bouton "Quitter"
    canvas.addEventListener("click", function (event) {
      let mouseX = event.clientX - canvas.offsetLeft;
      let mouseY = event.clientY - canvas.offsetTop;

      let buttonLeft = interfaceLeft + interfaceWidth / 2 - 50;
      let buttonTop = interfaceTop + interfaceHeight * 0.6;
      let buttonWidth = 100;
      let buttonHeight = 50;
      if (
        mouseX >= buttonLeft &&
        mouseX <= buttonLeft + buttonWidth &&
        mouseY >= buttonTop &&
        mouseY <= buttonTop + buttonHeight
      ) {
        router.ChangePages(routes[1].path);
        console.log("click");
      }
    });
  }

  const socket: Socket = io();

  let sushis: Sushi[] = [];
  socket.on("sushis", (data) => {
    for (let i = 0; i < data?.length; i++) {
      const sushi = new Sushi(
        data[i].x * (xEnd - xStart) + xStart,
        data[i].y * (-yStart + yEnd) + yStart,
        5
      );
      sushis.push(sushi);
    }
  });

  let folder = playerCharacter;
  let userId;

  function getUserId() {
    return new Promise((resolve, reject) => {
      socket.on("user", (data) => {
        userId = data;
        resolve(userId);
      });
    });
  }

  async function myFunction() {
    try {
      const userId = (await getUserId()) as number;
      let player = new Player(
        Math.random() * (xEnd - xStart) + xStart,
        Math.random() * (-yStart + yEnd) + yStart,
        20,
        folder,
        userId,
        1,
        playerName
      );

      socket.emit("newUser", player);
      let allUsers: Player[] = [];
      socket.on("allUsers", (data) => {
        allUsers = data.users;
        if (player && data.player) {
          player.x = data.player.x;
          player.y = data.player.y;
        }
      });

      socket.on("isDead", (data) => {
        if (data.winner.id == player.id) {
          player.radius += data.loser.radius;
        }
        if (data.loser.id == player.id) {
          player.isDead = true;
          console.log("you are dead");
        }
      });

      socket.on("newSushi", (data) => {
        sushis.splice(data.drop, 1);
        sushis.push(
          new Sushi(
            data.new.x * (xEnd - xStart) + xStart,
            data.new.y * (-yStart + yEnd) + yStart,
            5
          )
        );
      });

      let mouseX: number = 0;
      let mouseY: number = 0;

      canvas.addEventListener("mousemove", (event) => {
        event.preventDefault();
        mouseX = event.clientX;
        mouseY = event.clientY;
      });

      function update() {
        ctx.save();
        ctx.clearRect(0, 0, ctxWidth, ctxHeight);
        ctx.beginPath();
        const cameraX = canvas.width / 2 - player.x;
        const cameraY = canvas.height / 2 - player.y;
        ctx.translate(cameraX, cameraY);
        ctx.drawImage(backgroundImage, 0, 0, ctxWidth, ctxHeight);

        sushis.forEach((sushi) => {
          sushi.draw(ctx);
        });
        allUsers.forEach((user) => {
          const otherPlayer = new Player(
            user.x,
            user.y,
            user.radius,
            user.imageFolder,
            user.id,
            user.imageId,
            user.name
          );
          otherPlayer.draw(ctx);
        });
        if (player.isDead) {
          youAreDead(player);
        }
        socket.emit("userMoveTo", {
          player: player,
          mouseX: mouseX,
          mouseY: mouseY,
          canvasWidth: canvas.width,
          canvasHeight: canvas.height,
          ctxWidth: ctxWidth,
          ctxHeight: ctxHeight,
        });
        //player.draw(ctx);
        player.eatSushi(sushis, socket);
        ctx.restore();
        requestAnimationFrame(update);
      }
      update();
    } catch (error) {
      console.error(error);
    }
  }
  myFunction();
};

export default Main;
