import express from "express";
import http from "http";
const app = express();
const httpServer = http.createServer(app);
import { Server as IOServer } from "socket.io";
import Player from "./common/Player";
import { Sushi } from "./common/Ressources";

app.use("/", express.static("client/public"));

let users: Player[] = [];

let sushis: Partial<Sushi>[] = [];
for (let i = 0; i < 900; i++) {
  sushis.push({ x: Math.random(), y: Math.random() }); //commit "implement kill Player" et push
}

//Player function
function moveTo(
  player: Player,
  x: number,
  y: number,
  canvasWidth: number,
  canvasHeight: number
) {
  const directionX = x - canvasWidth / 2;
  const directionY = y - canvasHeight / 2;

  const distance = Math.sqrt(directionX ** 2 + directionY ** 2);
  player.direction.x = directionX / distance;
  player.direction.y = directionY / distance;
}

function move(
  player: Player,
  mouseX: number,
  mouseY: number,
  canvasWidth: number,
  canvasHeight: number
) {
  if (
    mouseX > canvasWidth / 2 - player.radius &&
    mouseX < canvasWidth / 2 + player.radius &&
    mouseY > canvasHeight / 2 - player.radius &&
    mouseY < canvasHeight / 2 + player.radius
  ) {
    return;
  }
  player.x += player.direction.x * player.speed;
  player.y += player.direction.y * player.speed;
  console.log(player.x);
  for (let i = 0; i < users.length; i++) {
    if (player.id === users[i].id) {
      users[i] = player;
    }
  }
  console.log(player.x, canvasWidth, canvasHeight);
  if (player.x < canvasWidth / 2) player.x = canvasWidth / 2;
  if (player.y < canvasHeight / 2) player.y = canvasHeight / 2;
  if (player.x > 6000 - canvasWidth / 2) {
    player.x = 6000 - canvasWidth / 2;
  }
  if (player.y > 3349 - canvasHeight / 2) {
    player.y = 3340 - canvasHeight / 2;
  }
}

function killPlayer(MyPlayer: Player) {
  let deadPlayer;
  for (let i = 0; i < users.length; i++) {
    const player = users[i];
    const dx = player.x - MyPlayer.x;
    const dy = player.y - MyPlayer.y;
    const distance = Math.sqrt(dx * dx + dy * dy);
    if (
      MyPlayer.radius > player.radius &&
      distance < MyPlayer.radius &&
      player.id != MyPlayer.id &&
      !MyPlayer.isDead
    ) {
      deadPlayer = player;
      users.splice(i, 1);
      break;
    }
  }
  return deadPlayer;
}

const io = new IOServer(httpServer);
io.on("connection", (socket) => {
  io.emit("user", socket.id);
  socket.emit("sushis", sushis);

  socket.on("newUser", (data) => {
    users.push(data);
    socket.emit("allUsers", { users: users });
  });

  socket.on("userMoveTo", (data) => {
    let player = data.player;
    console.log(data);
    moveTo(
      player,
      data.mouseX,
      data.mouseY,
      data.canvasWidth,
      data.canvasHeight
    );
    move(player, data.mouseX, data.mouseY, data.canvasWidth, data.canvasHeight);
    let deadPlayer = killPlayer(player);
    users.sort((a, b) => a.radius - b.radius);
    io.emit("allUsers", {
      users: users,
      player: player,
    });
    if (deadPlayer) {
      io.emit("isDead", { loser: deadPlayer, winner: player });
    }
  });

  socket.on("updateSushi", (data) => {
    sushis.splice(data, 1);
    sushis.push({ x: Math.random(), y: Math.random() });
    io.emit("newSushi", { new: sushis[sushis.length - 1], drop: data });
  });
});

const port = 8000;
httpServer.listen(port, () => {
  console.log(`Think http://localhost:${port}, Think !/`);
});
